import argparse
import os


def checks_if_file_is_readable(args):
    '''Cheks files's permission and existanse'''
    for file in args.files:
        if not os.path.exists(file):
            raise FileNotFoundError(f'file {file} does not exist')
        if not os.access(file, os.R_OK):
            raise PermissionError(f'file: "{file}", is not readable!')


def get_files_and_flags_from_cmd():
    '''Gets three files, order is described in help, two mutually exclusive flags (--asc is by defult True) and optional
     flag --driver, taking racer's full name. Returns all mentioned'''
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--asc', action='store_true')
    group.add_argument('--desc', action='store_true')
    parser.add_argument('-f', '--files', nargs=3,
                        help='files ordere: path_to_file_with_abbreviations, path_to_file_with_star_time, path_to_file_with_end_time)')
    parser.add_argument('-d', '--driver')
    args = parser.parse_args()
    # asc = True if neither --asc nor --desc
    if not args.asc and not args.desc:
        args.asc = True
    checks_if_file_is_readable(args)
    print(type(args))
    return args



from build_report import const_for_join


def check_if_driver_exists(dict_to_be_printed, args):
    'Looks for driver in list of names taken from dict'
    list_with_names = []
    for inner_list in dict_to_be_printed.values():
        list_with_names.append(inner_list[0])
    if args.driver not in list_with_names:
        raise ValueError(f'There is no driver named "{args.driver}"!')


def string_to_print_preparer(list_with_items_to_be_printed):
    '''Returns str (joined list) with name, comand, time devided by '|'
    Brendon Hartley|SCUDERIA TORO ROSSO HONDA|0:01:13.179000'''
    list_with_info_to_print = []
    for item in list_with_items_to_be_printed:
        list_with_info_to_print.append(str(item))
    string_to_be_printed = const_for_join.join(list_with_info_to_print)
    return string_to_be_printed



from datetime import datetime, timedelta
import logging
import warnings
from . import constants


def check():
    print(type('check'))
    print(type('check'))


def inserts_info_into_ordered_racers_dict(time_list, dict_with_racers_info):
    '''Return dict which keys are places(1-20) and values are list [driver's name, comand, time]. Since time_list is
    ordered, for each time func finds racer with same time, then adds info needed to be print in ordered_recers_dict '''
    ordered_recers_dict = {}
    place = 0
    list_with_used_keys = []
    for time in time_list:
        for abbrev in dict_with_racers_info:
            if str(dict_with_racers_info[abbrev][constants.const_absolute_time]) == str(
                    time) and abbrev not in list_with_used_keys:
                # Pice of code(shit) above is just enoying! Is there any stuf to skip current key after passing it to my dict?
                # Do not replace place += 1, becouse dict is looping till the end, so func will return only last racer
                # with datetime.timedelta(0)
                place += 1
                ordered_recers_dict[abbrev] = [dict_with_racers_info[abbrev][constants.const_name],
                                                   dict_with_racers_info[abbrev][constants.const_comand],
                                                   place,
                                                   str(dict_with_racers_info[abbrev][constants.const_absolute_time]),
                                                   dict_with_racers_info[abbrev][constants.const_start_time],
                                                   dict_with_racers_info[abbrev][constants.const_end_time]]
                list_with_used_keys.append(abbrev)
    return ordered_recers_dict


def time_counter(dict_with_racers_info):
    '''Add inner dict to each abrev consisting of key 'absolute_time' and value racer's absolute_time,
        which is timedelta obj, warning is raised when start_time is bigger than end_time, racer who has invalid time
        gets timedelta(0) which is zero'''
    for abbrev, inner_dict in dict_with_racers_info.items():
        start_time = datetime.strptime(inner_dict[constants.const_start_time], constants.const_time_format)
        end_time = datetime.strptime(inner_dict[constants.const_end_time], constants.const_time_format)
        if start_time > end_time:
            warnings.showwarning(
                f'Racer "{dict_with_racers_info[abbrev][constants.const_name]}", abrev - "{abbrev}", has invalid time!',
                UserWarning, filename='processing_dicts.py', lineno=35)
            invalid_time = timedelta(days=0, hours=0, minutes=0, milliseconds=0, microseconds=0)
            dict_with_racers_info[abbrev][constants.const_absolute_time] = invalid_time
        elif start_time < end_time:
            absolute_time = end_time - start_time
            dict_with_racers_info[abbrev][constants.const_absolute_time] = absolute_time
    return dict_with_racers_info


def time_snatcher(dict_with_racers_info, args):
    '''Returns list with ordered, depending on flags, time. Each time is timedelta obj!'''
    list_with_absolute_time = []
    for inner_dict in dict_with_racers_info.values():
        list_with_absolute_time.append(inner_dict[constants.const_absolute_time])
    if args.asc:
        list_with_absolute_time = sorted(list_with_absolute_time)
    elif args.desc:
        list_with_absolute_time = sorted(list_with_absolute_time, reverse=True)
    return list_with_absolute_time



from . import get_info_from_files, processing_dicts, parser


def build_report():
    '''Return dict which keys are places(1-20) and values are list [driver's name, comand, time], and args from cmd '''
    args = parser.get_files_and_flags_from_cmd()
    racers_info_dict = {}
    racers_info_dict = get_info_from_files.return_dict_with_time_and_racer_abbreviations(racers_info_dict,
                                                                                         args.files[1],
                                                                                         args.files[2])
    racers_info_dict = get_info_from_files.abbreviations_decoder(racers_info_dict, args.files[0])
    racers_info_dict = processing_dicts.time_counter(racers_info_dict)
    ordered_time_list = processing_dicts.time_snatcher(racers_info_dict, args)
    ordered_recers_dict = processing_dicts.inserts_info_into_ordered_racers_dict(ordered_time_list, racers_info_dict)
    return ordered_recers_dict, args



const_name = 'name'
const_comand = 'comand'
const_start_time = 'start time'
const_end_time = 'end time'
const_absolute_time = 'absolute time'
const_time_format = '%M:%S.%f'
const_for_join = '|'
const_divider = '_'
const_divider_length = 72
const_predivided_racers = 16




from build_report import build_report
from build_report import constants
import prepare_data_to_be_print


def print_report():
    '''Checks if driver was specifided, then calls proper func'''
    dict_to_be_printed, args = build_report()
    if args.driver:
        print_report_if_args_driver(dict_to_be_printed, args)
    else:
        print_report_if_no_args_provided(dict_to_be_printed)


def print_report_if_args_driver(dict_to_be_printed, args):
    '''Finds proper driver in the dict, prits his info.
    Dict keys are places(1-20) and values are list [driver's name, comand, time]'''
    prepare_data_to_be_print.check_if_driver_exists(dict_to_be_printed, args)
    for list_with_items_to_be_printed in dict_to_be_printed.values():
        if list_with_items_to_be_printed[0] == args.driver:
            list_with_info_to_print = []
            string_to_be_printed = prepare_data_to_be_print.string_to_print_preparer(list_with_items_to_be_printed)
            print(string_to_be_printed)


def print_report_if_no_args_provided(dict_to_be_printed):
    '''Prints first 16 drivers, then '______________' and other drivers
    Dict keys are places(1-20) and values are list [driver's name, comand, time]'''
    for place, list_with_items_to_be_printed in dict_to_be_printed.items():
        string_to_print = prepare_data_to_be_print.string_to_print_preparer(list_with_items_to_be_printed)
        if int(place) == constants.const_predivided_racers:
            print(constants.const_divider * constants.const_divider_length)
        print(string_to_print)


if __name__ == '__main__':
    print_report()



from . import constants


def abbreviations_decoder(dict_with_racers_info, path_to_file_with_racers_info):
    '''Add inner dict to each abrev consisting of keys 'name', 'comand' and values racer's name, racer's comand'''
    list_with_racers_names = []
    with open(path_to_file_with_racers_info, 'r', encoding='utf-8') as file:
        for line in file:
            if not line.isspace():
                line = line.split('_')
                abbreviation, name, comand = line
                dict_with_racers_info[abbreviation][constants.const_name] = name.strip()
                dict_with_racers_info[abbreviation][constants.const_comand] = comand.strip()
    return dict_with_racers_info


def return_dict_with_time_and_racer_abbreviations(dict_with_racers_info, path_to_file_with_star_time_and_abreviations,
                                                  path_to_file_with_end_time_and_abreviations):
    '''One_by-one read files with racers' time, return dict which keys are abreviations values are inner dicts with
    start/end time keys and values start/end time in str format'''
    path_to_files_with_time_and_abreviations = [path_to_file_with_star_time_and_abreviations,
                                                path_to_file_with_end_time_and_abreviations]
    for i, path_to_file_with_time_and_abreviations in enumerate(path_to_files_with_time_and_abreviations, start=1):
        with open(path_to_file_with_time_and_abreviations, 'r') as file:
            for line in file:
                if not line.isspace():
                    abbreviation, date, racer_time = get_time_date_and_racer_abbreviations_from_line(line)
                    if i == 1:
                        dict_with_racers_info[abbreviation] = {constants.const_start_time: racer_time.strip()}
                    elif i == 2:
                        dict_with_racers_info[abbreviation][constants.const_end_time] = racer_time.strip()
    return dict_with_racers_info


def get_time_date_and_racer_abbreviations_from_line(line_to_process):
    '''Reads file, returns abbreviation, date, racer_time in str format'''
    # 'i' is used to identify wether char belongs to abbreviation(chars 1-3), date(chars 4-16) or time(chars > 17)
    abbreviation, date, racer_time = '', '', ''
    for i, char in enumerate(line_to_process, start=1):
        if i <= 3:
            abbreviation += char
        elif i <= 16:
            date += char
        elif i > 17:
            racer_time += char
    return abbreviation, date, racer_time
